@@ -29,11 +29,12 @@ to "print for_debug" without much hassle (braces).
 from __future__ import print_function
 
 import os
+import struct
 import sys
+import textwrap
 import traceback
-from contextlib import contextmanager
 
-from nuitka.utils.ThreadedExecutor import RLock
+from nuitka.utils.Utils import isWin32Windows
 
 # Written by Options module.
 is_quiet = False
@@ -69,7 +70,7 @@ def flushStandardOutputs():
     sys.stderr.flush()
 
 
-def getEnableStyleCode(style):
+def _getEnableStyleCode(style):
     if style == "pink":
         style = "\033[95m"
     elif style == "blue":
@@ -107,21 +108,125 @@ def _enableAnsi():
         _enabled_ansi = True
 
 
-def getDisableStyleCode():
+def _getDisableStyleCode():
     return "\033[0m"
 
 
-# Locking seems necessary to avoid colored output split up.
-trace_lock = RLock()
+def _getIoctlGWINSZ(fd):
+    # On non-Windows, this might work, # spell-checker: ignore GWINSZ,TIOCGWINSZ
+    try:
+        # pylint: disable=I0021,import-error
+        import fcntl
+        import termios
+
+        return struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ, '1234'))
+    except BaseException:  # Catch all the things, pylint: disable=broad-except
+        return None
 
 
-@contextmanager
-def withTraceLock():
-    """Hold a lock, so traces cannot be output at the same time mixing them up."""
+def _getTerminalSizeWin32():
+    try:
+        from ctypes import create_string_buffer, windll
 
-    trace_lock.acquire()
-    yield
-    trace_lock.release()
+        # handles are -10 to -12
+        h = windll.kernel32.GetStdHandle(-12)
+        buffer = create_string_buffer(22)
+
+        res = windll.kernel32.GetConsoleScreenBufferInfo(h, buffer)
+        if res:
+            # TODO: Could also define the C structure
+            (
+                _,
+                _,
+                _,
+                _,
+                _,
+                left,
+                top,
+                right,
+                bottom,
+                _,
+                _,
+            ) = struct.unpack("hhhhHhhhhhh", buffer.raw)
+
+            columns = right - left + 1
+            lines = bottom - top + 1
+
+            return lines, columns
+    except BaseException:  # Catch all the things, pylint: disable=broad-except
+        return None
+
+
+def _getTerminalSize():
+    # For Python2, we get to do it ourselves.
+
+    if str is bytes:
+        if isWin32Windows():
+            columns = _getTerminalSizeWin32()
+        else:
+            # Try all file handles.
+            columns = _getIoctlGWINSZ(0) or _getIoctlGWINSZ(1) or _getIoctlGWINSZ(2)
+
+        if columns:
+            return columns[1]
+
+        try:
+            return int(os.environ.get("COLUMNS", "1000"))
+        except ValueError:
+            return 1000
+    else:
+        try:
+            return os.get_terminal_size()[0]
+        except OSError:
+            return 1000
+
+
+def aliasStyle(style):
+    if style == "test-prepare":
+        return "pink"
+    if style == "test-progress":
+        return "blue"
+    if style == "test-debug":
+        return "bold"
+    else:
+        return style
+
+
+def _my_print(file_output, is_atty, args, kwargs):
+    if "style" in kwargs:
+        style = kwargs["style"]
+
+        style = aliasStyle(style)
+
+        del kwargs["style"]
+
+        if "end" in kwargs:
+            end = kwargs["end"]
+            del kwargs["end"]
+        else:
+            end = "\n"
+
+        if style is not None and is_atty:
+            enable_style = _getEnableStyleCode(style)
+
+            if enable_style is None:
+                raise ValueError(
+                    "%r is an invalid value for keyword argument style" % style
+                )
+
+            _enableAnsi()
+
+            print(enable_style, end="", **kwargs)
+
+        print(*args, end=end, **kwargs)
+
+        if style is not None and is_atty:
+            print(_getDisableStyleCode(), end="", **kwargs)
+    else:
+        print(*args, **kwargs)
+
+    # Flush the output.
+    file_output.flush()
 
 
 def my_print(*args, **kwargs):
@@ -136,43 +241,10 @@ def my_print(*args, **kwargs):
     is_atty = file_output.isatty()
 
     if progress and is_atty:
-        progress.hideProgressBar()
-
-    with withTraceLock():
-        if "style" in kwargs:
-            style = kwargs["style"]
-            del kwargs["style"]
-
-            if "end" in kwargs:
-                end = kwargs["end"]
-                del kwargs["end"]
-            else:
-                end = "\n"
-
-            if style is not None and is_atty:
-                enable_style = getEnableStyleCode(style)
-
-                if enable_style is None:
-                    raise ValueError(
-                        "%r is an invalid value for keyword argument style" % style
-                    )
-
-                _enableAnsi()
-
-                print(enable_style, end="", **kwargs)
-
-            print(*args, end=end, **kwargs)
-
-            if style is not None and is_atty:
-                print(getDisableStyleCode(), end="", **kwargs)
-        else:
-            print(*args, **kwargs)
-
-        # Flush the output.
-        file_output.flush()
-
-    if progress and is_atty:
-        progress.resumeProgressBar()
+        with progress.withExternalWritingPause():
+            _my_print(file_output, is_atty, args, kwargs)
+    else:
+        _my_print(file_output, is_atty, args, kwargs)
 
 
 class OurLogger(object):
@@ -191,19 +263,34 @@ class OurLogger(object):
     def warning(self, message, style="red"):
         if not self.is_no_warnings:
             if self.name:
-                message = "%s:WARNING: %s" % (self.name, message)
+                prefix = "%s:WARNING: " % self.name
             else:
-                message = "WARNING: %s" % message
+                prefix = "WARNING: "
 
             style = style or self.base_style
-            self.my_print(message, style=style, file=sys.stderr)
 
-    def sysexit(self, message, exit_code=1):
+            if sys.stderr.isatty():
+                width = _getTerminalSize()
+            else:
+                width = 10000
+
+            formatted_message = textwrap.fill(
+                message,
+                width=width,
+                initial_indent=prefix,
+                subsequent_indent=prefix,
+                break_on_hyphens=False,
+                expand_tabs=False,
+            )
+            self.my_print(formatted_message, style=style, file=sys.stderr)
+
+    def sysexit(self, message="", exit_code=1):
         from nuitka.Progress import closeProgressBar
 
         closeProgressBar()
 
-        self.my_print("FATAL: %s" % message, style="red", file=sys.stderr)
+        if message:
+            self.my_print("FATAL: %s" % message, style="red", file=sys.stderr)
 
         sys.exit(exit_code)
 
@@ -232,8 +319,6 @@ class FileLogger(OurLogger):
         self.file_handle = file_handle
 
     def my_print(self, message, **kwargs):
-        message = message + "\n"
-
         if "file" not in kwargs:
             kwargs["file"] = self.file_handle or sys.stdout
 
@@ -243,6 +328,9 @@ class FileLogger(OurLogger):
     def setFileHandle(self, file_handle):
         self.file_handle = file_handle
 
+    def isFileOutput(self):
+        return self.file_handle is not None
+
     def info(self, message, style=None):
         if not self.isQuiet() or self.file_handle:
             message = "%s:INFO: %s" % (self.name, message)
@@ -266,11 +354,12 @@ class FileLogger(OurLogger):
 
 general = OurLogger("Nuitka")
 plugins_logger = OurLogger("Nuitka-Plugins")
-recursion_logger = OurLogger("Nuitka-Recursion")
+recursion_logger = OurLogger("Nuitka-Inclusion")
 progress_logger = OurLogger("Nuitka-Progress", quiet=True)
 memory_logger = OurLogger("Nuitka-Memory")
 dependencies_logger = OurLogger("Nuitka-Dependencies")
 optimization_logger = FileLogger("Nuitka-Optimization")
+pgo_logger = FileLogger("Nuitka-PGO")
 codegen_logger = OurLogger("Nuitka-Codegen")
 inclusion_logger = FileLogger("Nuitka-Inclusion")
 scons_logger = OurLogger("Nuitka-Scons")
@@ -280,3 +369,5 @@ options_logger = OurLogger("Nuitka-Options")
 unusual_logger = OurLogger("Nuitka-Unusual")
 datacomposer_logger = OurLogger("Nuitka-Datacomposer")
 onefile_logger = OurLogger("Nuitka-Onefile")
+tools_logger = OurLogger("Nuitka-Tools")
+wheel_logger = OurLogger("Nuitka-Wheel")
