@@ -22,11 +22,11 @@ directory and then compiles that to either an executable or an extension module
 or package, that can contain all used modules too.
 """
 
-# Import as little as possible initially, because we might be re-executing
-# soon.
+# Note: This avoids imports at all costs, such that initial startup doesn't do more
+# than necessary, until re-execution has been decided.
+
 import os
 import sys
-import warnings
 
 
 def main():
@@ -54,11 +54,6 @@ def main():
             if os.path.dirname(os.path.abspath(__file__)) != path_element
         ]
 
-    # We don't care, and these are triggered by run time calculations of "range" and
-    # others, while on python2.7 they are disabled by default.
-
-    warnings.simplefilter("ignore", DeprecationWarning)
-
     # We will run with the Python configuration as specified by the user, if it does
     # not match, we restart ourselves with matching configuration.
     needs_reexec = False
@@ -74,68 +69,23 @@ def main():
     if os.environ.get("PYTHONHASHSEED", "-1") != "0":
         needs_reexec = True
 
-    # For re-execution, we might not have done this.
-    from nuitka import Options  # isort:skip
-
     # In case we need to re-execute.
     if needs_reexec:
-        # TODO: If that's the only one, why do it at all..
-        Options.parseArgs(will_reexec=True)
+        from nuitka.utils.ReExecute import reExecuteNuitka  # isort:skip
 
-        if not Options.isAllowedToReexecute():
-            sys.exit("Error, not allowed to re-execute, but that would be needed.")
+        # Does not return
+        reExecuteNuitka(pgo_filename=None)
 
-        our_filename = sys.modules[__name__].__file__
+    # We don't care about deprecations in any version, and these are triggered
+    # by run time calculations of "range" and others, while on python2.7 they
+    # are disabled by default.
+    import warnings
 
-        # Execute with full path as the process name, so it can find itself and its
-        # libraries.
-        args = [sys.executable, sys.executable]
+    warnings.simplefilter("ignore", DeprecationWarning)
 
-        from nuitka.PythonVersions import python_version
+    from nuitka import Options  # isort:skip
 
-        if python_version >= 0x370 and sys.flags.utf8_mode:
-            args += ["-X", "utf8"]
-
-        args += ["-S", our_filename]
-
-        os.environ["NUITKA_BINARY_NAME"] = sys.modules["__main__"].__file__
-        os.environ["NUITKA_PACKAGE_HOME"] = os.path.dirname(
-            os.path.abspath(sys.modules["nuitka"].__path__[0])
-        )
-
-        if Options.is_nuitka_run:
-            args.append("--run")
-
-        # Same arguments as before.
-        args += sys.argv[1:] + list(Options.getMainArgs())
-
-        os.environ["NUITKA_PYTHONPATH"] = repr(sys.path)
-
-        from nuitka.importing.PreloadedPackages import (
-            detectPreLoadedPackagePaths,
-            detectPthImportedPackages,
-        )
-
-        os.environ["NUITKA_NAMESPACES"] = repr(detectPreLoadedPackagePaths())
-
-        if "site" in sys.modules:
-            os.environ["NUITKA_SITE_FILENAME"] = sys.modules["site"].__file__
-
-            os.environ["NUITKA_PTH_IMPORTED"] = repr(detectPthImportedPackages())
-
-        os.environ["NUITKA_SITE_FLAG"] = (
-            str(sys.flags.no_site) if not Options.hasPythonFlagNoSite() else "1"
-        )
-
-        os.environ["PYTHONHASHSEED"] = "0"
-
-        os.environ["NUITKA_REEXECUTION"] = "1"
-
-        from nuitka.utils import Execution  # isort:skip
-
-        Execution.callExec(args)
-
-    Options.parseArgs(will_reexec=False)
+    Options.parseArgs()
 
     Options.commentArgs()
 
