@@ -26,6 +26,12 @@ branches and make a code block out of it. But it doesn't contain any target
 language syntax.
 """
 
+from nuitka.nodes.AttributeNodesGenerated import (
+    attribute_classes,
+    attribute_typed_classes,
+)
+from nuitka.nodes.BytesNodes import getBytesOperationClasses
+from nuitka.nodes.StrNodes import getStrOperationClasses
 from nuitka.plugins.Plugins import Plugins
 from nuitka.utils.CStrings import encodePythonStringToC
 
@@ -75,11 +81,13 @@ from .BuiltinCodes import (
 )
 from .CallCodes import generateCallCode, getCallsCode
 from .ClassCodes import generateBuiltinSuperCode, generateSelectMetaclassCode
-from .CodeHelpers import setExpressionDispatchDict, setStatementDispatchDict
+from .CodeHelpers import addExpressionDispatchDict, setStatementDispatchDict
 from .ComparisonCodes import (
     generateBuiltinIsinstanceCode,
     generateBuiltinIssubclassCode,
     generateComparisonExpressionCode,
+    generateMatchTypeCheckMappingCode,
+    generateMatchTypeCheckSequenceCode,
     generateTypeCheckCode,
 )
 from .ConditionalCodes import (
@@ -102,12 +110,31 @@ from .CoroutineCodes import (
 from .DictCodes import (
     generateBuiltinDictCode,
     generateDictionaryCreationCode,
-    generateDictOperationGetCode,
+    generateDictOperationClearCode,
+    generateDictOperationCopyCode,
+    generateDictOperationGet2Code,
+    generateDictOperationGet3Code,
     generateDictOperationInCode,
+    generateDictOperationItemCode,
+    generateDictOperationItemsCode,
+    generateDictOperationIteritemsCode,
+    generateDictOperationIterkeysCode,
+    generateDictOperationItervaluesCode,
+    generateDictOperationKeysCode,
+    generateDictOperationPop2Code,
+    generateDictOperationPop3Code,
     generateDictOperationRemoveCode,
     generateDictOperationSetCode,
     generateDictOperationSetCodeKeyValue,
+    generateDictOperationSetdefault2Code,
+    generateDictOperationSetdefault3Code,
+    generateDictOperationUpdate2Code,
+    generateDictOperationUpdate3Code,
     generateDictOperationUpdateCode,
+    generateDictOperationValuesCode,
+    generateDictOperationViewitemsCode,
+    generateDictOperationViewkeysCode,
+    generateDictOperationViewvaluesCode,
 )
 from .EvalCodes import (
     generateBuiltinCompileCode,
@@ -156,11 +183,15 @@ from .GlobalsLocalsCodes import (
 from .IdCodes import generateBuiltinHashCode, generateBuiltinIdCode
 from .ImportCodes import (
     generateBuiltinImportCode,
+    generateConstantSysVersionInfoCode,
+    generateImportlibImportCallCode,
+    generateImportModuleFixedCode,
     generateImportModuleHardCode,
     generateImportModuleNameHardCode,
     generateImportNameCode,
     generateImportStarCode,
 )
+from .InjectCCodes import generateInjectCCode
 from .IntegerCodes import (
     generateBuiltinInt1Code,
     generateBuiltinInt2Code,
@@ -200,6 +231,7 @@ from .LoopCodes import (
     generateLoopCode,
     generateLoopContinueCode,
 )
+from .MatchCodes import generateMatchArgsCode
 from .ModuleCodes import (
     generateModuleAttributeCode,
     generateModuleAttributeFileCode,
@@ -211,6 +243,13 @@ from .OperationCodes import (
     generateOperationNotCode,
     generateOperationUnaryCode,
 )
+from .PackageResourceCodes import (
+    generateImportlibResourcesReadBinaryCallCode,
+    generateImportlibResourcesReadTextCallCode,
+    generatePkglibGetDataCallCode,
+    generatePkgResourcesResourceStreamCallCode,
+    generatePkgResourcesResourceStringCallCode,
+)
 from .PrintCodes import generatePrintNewlineCode, generatePrintValueCode
 from .RaisingCodes import (
     generateRaiseCode,
@@ -249,11 +288,14 @@ from .StringCodes import (
     generateBuiltinOrdCode,
     generateBuiltinStrCode,
     generateBuiltinUnicodeCode,
+    generateBytesOperationCode,
     generateStringContenationCode,
+    generateStrOperationCode,
 )
 from .SubscriptCodes import (
     generateAssignmentSubscriptCode,
     generateDelSubscriptCode,
+    generateSubscriptCheckCode,
     generateSubscriptLookupCode,
 )
 from .TryCodes import generateTryCode
@@ -411,7 +453,7 @@ def generateFunctionBodyCode(function_body, context):
     return function_code, function_decl
 
 
-def generateModuleCode(module, data_filename):
+def _generateModuleCode(module, data_filename):
     # As this not only creates all modules, but also functions, it deals
     # also with its functions.
 
@@ -474,6 +516,13 @@ def generateModuleCode(module, data_filename):
     )
 
 
+def generateModuleCode(module, data_filename):
+    try:
+        return _generateModuleCode(module=module, data_filename=data_filename)
+    except KeyboardInterrupt:
+        raise KeyboardInterrupt("Interrupted while working on", module)
+
+
 def generateHelpersCode():
     calls_decl_code, calls_body_code = getCallsCode()
 
@@ -487,7 +536,7 @@ def generateHelpersCode():
     )
 
 
-setExpressionDispatchDict(
+addExpressionDispatchDict(
     {
         "EXPRESSION_ATTRIBUTE_CHECK": generateAttributeCheckCode,
         "EXPRESSION_ATTRIBUTE_LOOKUP": generateAttributeLookupCode,
@@ -548,6 +597,8 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_ISINSTANCE": generateBuiltinIsinstanceCode,
         "EXPRESSION_BUILTIN_ISSUBCLASS": generateBuiltinIssubclassCode,
         "EXPRESSION_TYPE_CHECK": generateTypeCheckCode,
+        "EXPRESSION_MATCH_TYPE_CHECK_SEQUENCE": generateMatchTypeCheckSequenceCode,
+        "EXPRESSION_MATCH_TYPE_CHECK_MAPPING": generateMatchTypeCheckMappingCode,
         "EXPRESSION_BUILTIN_DIR1": generateBuiltinDir1Code,
         "EXPRESSION_BUILTIN_VARS": generateBuiltinVarsCode,
         "EXPRESSION_BUILTIN_HASATTR": generateBuiltinHasattrCode,
@@ -606,6 +657,8 @@ setExpressionDispatchDict(
         "EXPRESSION_CONSTANT_TYPE_SUBSCRIPTABLE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_BYTEARRAY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_GENERIC_ALIAS": generateConstantGenericAliasCode,
+        "EXPRESSION_CONSTANT_UNION_TYPE": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_SYS_VERSION_INFO_REF": generateConstantSysVersionInfoCode,
         "EXPRESSION_CONDITIONAL": generateConditionalCode,
         "EXPRESSION_CONDITIONAL_OR": generateConditionalAndOrCode,
         "EXPRESSION_CONDITIONAL_AND": generateConditionalAndOrCode,
@@ -622,14 +675,38 @@ setExpressionDispatchDict(
         "EXPRESSION_COMPARISON_GTE": generateComparisonExpressionCode,
         "EXPRESSION_COMPARISON_EQ": generateComparisonExpressionCode,
         "EXPRESSION_COMPARISON_NEQ": generateComparisonExpressionCode,
-        "EXPRESSION_DICT_OPERATION_GET": generateDictOperationGetCode,
+        "EXPRESSION_DICT_OPERATION_ITEM": generateDictOperationItemCode,
+        "EXPRESSION_DICT_OPERATION_GET2": generateDictOperationGet2Code,
+        "EXPRESSION_DICT_OPERATION_GET3": generateDictOperationGet3Code,
+        "EXPRESSION_DICT_OPERATION_HASKEY": generateDictOperationInCode,
         "EXPRESSION_DICT_OPERATION_IN": generateDictOperationInCode,
         "EXPRESSION_DICT_OPERATION_NOT_IN": generateDictOperationInCode,
+        "EXPRESSION_DICT_OPERATION_COPY": generateDictOperationCopyCode,
+        "EXPRESSION_DICT_OPERATION_CLEAR": generateDictOperationClearCode,
+        "EXPRESSION_DICT_OPERATION_ITEMS": generateDictOperationItemsCode,
+        "EXPRESSION_DICT_OPERATION_ITERITEMS": generateDictOperationIteritemsCode,
+        "EXPRESSION_DICT_OPERATION_VIEWITEMS": generateDictOperationViewitemsCode,
+        "EXPRESSION_DICT_OPERATION_KEYS": generateDictOperationKeysCode,
+        "EXPRESSION_DICT_OPERATION_ITERKEYS": generateDictOperationIterkeysCode,
+        "EXPRESSION_DICT_OPERATION_VIEWKEYS": generateDictOperationViewkeysCode,
+        "EXPRESSION_DICT_OPERATION_VALUES": generateDictOperationValuesCode,
+        "EXPRESSION_DICT_OPERATION_ITERVALUES": generateDictOperationItervaluesCode,
+        "EXPRESSION_DICT_OPERATION_VIEWVALUES": generateDictOperationViewvaluesCode,
+        "EXPRESSION_DICT_OPERATION_SETDEFAULT2": generateDictOperationSetdefault2Code,
+        "EXPRESSION_DICT_OPERATION_SETDEFAULT3": generateDictOperationSetdefault3Code,
+        "EXPRESSION_DICT_OPERATION_POP2": generateDictOperationPop2Code,
+        "EXPRESSION_DICT_OPERATION_POP3": generateDictOperationPop3Code,
+        "EXPRESSION_DICT_OPERATION_UPDATE2": generateDictOperationUpdate2Code,
+        "EXPRESSION_DICT_OPERATION_UPDATE3": generateDictOperationUpdate3Code,
         "EXPRESSION_FUNCTION_CREATION": generateFunctionCreationCode,
         "EXPRESSION_FUNCTION_CALL": generateFunctionCallCode,
         "EXPRESSION_FUNCTION_ERROR_STR": generateFunctionErrorStrCode,
+        "EXPRESSION_IMPORT_MODULE_FIXED": generateImportModuleFixedCode,
         "EXPRESSION_IMPORT_MODULE_HARD": generateImportModuleHardCode,
-        "EXPRESSION_IMPORT_MODULE_NAME_HARD": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORT_MODULE_NAME_HARD_MAYBE_EXISTS": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORT_MODULE_NAME_HARD_EXISTS": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_IMPORT_MODULE_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_IMPORT_MODULE_CALL": generateImportlibImportCallCode,
         "EXPRESSION_IMPORT_NAME": generateImportNameCode,
         "EXPRESSION_LIST_OPERATION_EXTEND": generateListOperationExtendCode,
         "EXPRESSION_LIST_OPERATION_EXTEND_FOR_UNPACK": generateListOperationExtendCode,
@@ -687,6 +764,7 @@ setExpressionDispatchDict(
         # TODO: Rename to make more clear it is an outline
         "EXPRESSION_CLASS_BODY": generateFunctionOutlineCode,
         "EXPRESSION_SUBSCRIPT_LOOKUP": generateSubscriptLookupCode,
+        "EXPRESSION_SUBSCRIPT_CHECK": generateSubscriptCheckCode,
         "EXPRESSION_SLICE_LOOKUP": generateSliceLookupCode,
         "EXPRESSION_SET_OPERATION_UPDATE": generateSetOperationUpdateCode,
         "EXPRESSION_SIDE_EFFECTS": generateSideEffectsCode,
@@ -711,9 +789,41 @@ setExpressionDispatchDict(
         "EXPRESSION_LOCALS_VARIABLE_REF": generateLocalsDictVariableRefCode,
         "EXPRESSION_RAISE_EXCEPTION": generateRaiseExpressionCode,
         "EXPRESSION_NUITKA_LOADER_CREATION": generateNuitkaLoaderCreationCode,
+        "EXPRESSION_PKGLIB_GET_DATA_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKG_RESOURCES_RESOURCE_STRING_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKG_RESOURCES_RESOURCE_STREAM_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_RESOURCES_READ_BINARY_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_RESOURCES_READ_TEXT_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKGLIB_GET_DATA_CALL": generatePkglibGetDataCallCode,
+        "EXPRESSION_PKG_RESOURCES_RESOURCE_STRING_CALL": generatePkgResourcesResourceStringCallCode,
+        "EXPRESSION_PKG_RESOURCES_RESOURCE_STREAM_CALL": generatePkgResourcesResourceStreamCallCode,
+        "EXPRESSION_IMPORTLIB_RESOURCES_READ_BINARY_CALL": generateImportlibResourcesReadBinaryCallCode,
+        "EXPRESSION_IMPORTLIB_RESOURCES_READ_TEXT_CALL": generateImportlibResourcesReadTextCallCode,
+        "EXPRESSION_MATCH_ARGS": generateMatchArgsCode,
     }
 )
 
+# Add code generation for the EXPRESSION_ATTRIBUTE_LOOKUP_FIXED_* variety
+addExpressionDispatchDict(
+    dict((cls.kind, generateAttributeLookupCode) for cls in attribute_classes.values())
+)
+
+# Add code generation for the EXPRESSION_ATTRIBUTE_LOOKUP_DICT|LIST|STR_* variety
+addExpressionDispatchDict(
+    dict((cls.kind, generateAttributeLookupCode) for cls in attribute_typed_classes)
+)
+
+# Add code generation for the EXPRESSION_STR_OPERATION_* nodes.
+addExpressionDispatchDict(
+    dict((cls.kind, generateStrOperationCode) for cls in getStrOperationClasses())
+)
+
+# Add code generation for the EXPRESSION_BYTES_OPERATION_* nodes.
+addExpressionDispatchDict(
+    dict((cls.kind, generateBytesOperationCode) for cls in getBytesOperationClasses())
+)
+
+
 setStatementDispatchDict(
     {
         "STATEMENT_ASSIGNMENT_VARIABLE": generateAssignmentVariableCode,
@@ -762,5 +872,6 @@ setStatementDispatchDict(
         "STATEMENT_PRESERVE_FRAME_EXCEPTION": generateFramePreserveExceptionCode,
         "STATEMENT_RESTORE_FRAME_EXCEPTION": generateFrameRestoreExceptionCode,
         "STATEMENT_PUBLISH_EXCEPTION": generateExceptionPublishCode,
+        "STATEMENT_INJECT_C_CODE": generateInjectCCode,
     }
 )
