@@ -33,8 +33,10 @@ import sys
 from nuitka import Options
 from nuitka.constants.Serialization import ConstantAccessor
 from nuitka.PythonVersions import python_version
-from nuitka.Version import getNuitkaVersion
+from nuitka.Version import getNuitkaVersionTuple
 
+from .CodeHelpers import withObjectCodeTemporaryAssignment
+from .ErrorCodes import getAssertionCode
 from .GlobalConstants import getConstantDefaultPopulation
 from .Namify import namifyConstant
 from .templates.CodeTemplatesConstants import template_constants_reading
@@ -42,16 +44,50 @@ from .templates.CodeTemplatesModules import template_header_guard
 
 
 def generateConstantReferenceCode(to_name, expression, emit, context):
-    """ Assign the constant behind the expression to to_name."""
+    """Assign the constant behind the expression to to_name."""
 
     to_name.getCType().emitAssignmentCodeFromConstant(
         to_name=to_name,
         constant=expression.getCompileTimeConstant(),
+        # Derive this from context.
+        may_escape=True,
         emit=emit,
         context=context,
     )
 
 
+def generateConstantGenericAliasCode(to_name, expression, emit, context):
+    # TODO: Have these as prepared constants as well, if args are not mutable.
+
+    origin_name = context.allocateTempName("generic_alias_origin")
+    args_name = context.allocateTempName("generic_alias_args")
+
+    origin_name.getCType().emitAssignmentCodeFromConstant(
+        to_name=origin_name,
+        constant=expression.getCompileTimeConstant().__origin__,
+        may_escape=True,
+        emit=emit,
+        context=context,
+    )
+
+    args_name.getCType().emitAssignmentCodeFromConstant(
+        to_name=args_name,
+        constant=expression.getCompileTimeConstant().__args__,
+        may_escape=True,
+        emit=emit,
+        context=context,
+    )
+
+    with withObjectCodeTemporaryAssignment(
+        to_name, "builtin_value", expression, emit, context
+    ) as value_name:
+        emit("%s = Py_GenericAlias(%s, %s);" % (value_name, origin_name, args_name))
+
+        getAssertionCode(check="%s != NULL" % value_name, emit=emit)
+
+        context.addCleanupTempName(value_name)
+
+
 def getConstantsDefinitionCode():
     """Create the code code "__constants.c" and "__constants.h" files.
 
@@ -59,6 +95,8 @@ def getConstantsDefinitionCode():
     than one module) and create them.
 
     """
+    # Somewhat detail rich, pylint: disable=too-many-locals
+
     constant_accessor = ConstantAccessor(
         data_filename="__constants.const", top_level_name="global_constants"
     )
@@ -113,13 +151,7 @@ def getConstantsDefinitionCode():
         "header_body": "\n".join(lines),
     }
 
-    major, minor, micro = getNuitkaVersion().split(".")[:3]
-
-    if "rc" in micro:
-        micro = micro[: micro.find("rc")]
-        level = "candidate"
-    else:
-        level = "release"
+    major, minor, micro, is_final, _rc_number = getNuitkaVersionTuple()
 
     body = template_constants_reading % {
         "global_constants_count": constant_accessor.getConstantsCount(),
@@ -131,7 +163,7 @@ def getConstantsDefinitionCode():
         "nuitka_version_major": major,
         "nuitka_version_minor": minor,
         "nuitka_version_micro": micro,
-        "nuitka_version_level": level,
+        "nuitka_version_level": "release" if is_final else "candidate",
     }
 
     return header, body
