@@ -21,12 +21,10 @@
 
 import os
 import re
-import shutil
 
 from nuitka import Options
-from nuitka.freezer.IncludedEntryPoints import makeDllEntryPoint
 from nuitka.plugins.PluginBase import NuitkaPluginBase
-from nuitka.utils.FileOperations import getFileList
+from nuitka.utils.FileOperations import copyFile, getFileList
 from nuitka.utils.Importing import getSharedLibrarySuffix
 from nuitka.utils.Utils import isWin32Windows
 
@@ -64,7 +62,7 @@ class NuitkaPluginZmq(NuitkaPluginBase):
         if module_name == "zmq.libzmq" and isWin32Windows():
             # TODO: Very strange thing for zmq on Windows, needs the .pyd file in wrong dir too. Have
             # this done in a dedicated form somewhere.
-            shutil.copyfile(
+            copyFile(
                 os.path.join(dist_dir, "zmq\\libzmq.pyd"),
                 os.path.join(
                     dist_dir, "libzmq" + getSharedLibrarySuffix(preferred=True)
@@ -80,6 +78,8 @@ class NuitkaPluginZmq(NuitkaPluginBase):
 
     def onModuleSourceCode(self, module_name, source_code):
         if module_name == "zmq" and Options.isStandaloneMode():
+            # TODO: Make the anti-bloat engine to this.
+
             if "_delvewheel_init_patch" in source_code:
                 match = re.search(
                     r"(def _delvewheel_init_patch_(.*?)\(\):\n.*?_delvewheel_init_patch_\2\(\))",
@@ -100,7 +100,7 @@ class NuitkaPluginZmq(NuitkaPluginBase):
 
                 # Fake the __file__ to the proper value.
                 exec_globals = {
-                    "__file__": self.locateModule(None, module_name) + "\\__init__.py",
+                    "__file__": self.locateModule(module_name) + "\\__init__.py",
                     "add_dll_directory": self._add_dll_directory,
                 }
 
@@ -110,7 +110,7 @@ class NuitkaPluginZmq(NuitkaPluginBase):
     def getExtraDlls(self, module):
         if module.getFullName() == "zmq" and self.dll_directory is not None:
             for dll_filename in getFileList(self.dll_directory):
-                yield makeDllEntryPoint(
+                yield self.makeDllEntryPoint(
                     source_path=dll_filename,
                     dest_path=os.path.join(
                         "pyzmq.libs", os.path.basename(dll_filename)
