@@ -21,13 +21,13 @@
 
 """
 
-from __future__ import print_function
-
 import os
 import sys
 from optparse import OptionParser
 
+from nuitka.PythonVersions import isDebugPython
 from nuitka.tools.testing.Common import checkReferenceCount, getTempDir
+from nuitka.Tracing import my_print
 from nuitka.utils.Execution import check_call
 from nuitka.utils.Importing import importFileAsModule
 
@@ -64,7 +64,7 @@ Try to explain the differences by comparing object counts.""",
 
     # First with pure Python.
     checked_module = importFileAsModule(options.checked_module)
-    print("Using", checked_module.main)
+    my_print("Using %s" % checked_module.main, style="blue")
     checkReferenceCount(checked_module.main, explain=options.explain)
 
     temp_dir = getTempDir()
@@ -74,14 +74,12 @@ Try to explain the differences by comparing object counts.""",
         "nuitka",
         "--module",
         options.checked_module,
-        "--output-dir",
-        temp_dir,
+        "--output-dir=%s" % temp_dir,
     ]
 
-    if hasattr(sys, "gettotalrefcount"):
+    if isDebugPython():
         command.append("--python-debug")
 
-    # print(command)
     check_call(command)
 
     module_name = os.path.basename(options.checked_module).split(".")[0]
@@ -89,7 +87,7 @@ Try to explain the differences by comparing object counts.""",
     sys.path.insert(0, temp_dir)
     checked_module = __import__(module_name)
 
-    print("Using", checked_module.main)
+    my_print("Using %s" % checked_module.main, style="blue")
     checkReferenceCount(checked_module.main)
 
 
