@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,12 +17,10 @@
 #     limitations under the License.
 #
 
-""" Tool to compare output of CPython and Nuitka.
+""" Tool to compare find SxS using modules.
 
 """
 
-from __future__ import print_function
-
 import os
 import sys
 import tempfile
