@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,8 +17,8 @@
 #
 """ Reformulation of function statements.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
@@ -36,7 +36,7 @@ from nuitka.nodes.BuiltinIteratorNodes import (
     StatementSpecialUnpackCheck,
 )
 from nuitka.nodes.BuiltinNextNodes import ExpressionSpecialUnpack
-from nuitka.nodes.BuiltinRefNodes import makeExpressionBuiltinRef
+from nuitka.nodes.BuiltinRefNodes import makeExpressionBuiltinTypeRef
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
 from nuitka.nodes.CoroutineNodes import (
     ExpressionCoroutineObjectBody,
@@ -60,6 +60,8 @@ from nuitka.nodes.VariableRefNodes import (
     ExpressionVariableNameRef,
     ExpressionVariableRef,
 )
+from nuitka.Options import hasPythonFlagNoAnnotations
+from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 from nuitka.specs.ParameterSpecs import ParameterSpec
 
@@ -86,8 +88,9 @@ def _insertFinalReturnStatement(function_statements_body, return_statement):
             statement=return_statement
         )
     elif not function_statements_body.isStatementAborting():
-        function_statements_body.setStatements(
-            function_statements_body.getStatements() + (return_statement,)
+        function_statements_body.setChild(
+            "statements",
+            function_statements_body.subnode_statements + (return_statement,),
         )
 
     return function_statements_body
@@ -95,8 +98,7 @@ def _insertFinalReturnStatement(function_statements_body, return_statement):
 
 def _insertInitialSetLocalsDictStatement(function_body, function_statements_body):
     locals_statement = StatementSetLocalsDictionary(
-        locals_scope=function_body.getFunctionLocalsScope(),
-        source_ref=function_body.source_ref,
+        locals_scope=function_body.getLocalsScope(), source_ref=function_body.source_ref
     )
 
     if function_statements_body is None:
@@ -104,8 +106,9 @@ def _insertInitialSetLocalsDictStatement(function_body, function_statements_body
             statement=locals_statement
         )
     else:
-        function_statements_body.setStatements(
-            (locals_statement,) + function_statements_body.getStatements()
+        function_statements_body.setChild(
+            "statements",
+            (locals_statement,) + function_statements_body.subnode_statements,
         )
 
     return function_statements_body
@@ -123,7 +126,7 @@ def _injectDecorator(decorators, inject, acceptable, source_ref):
             break
     else:
         decorators.append(
-            makeExpressionBuiltinRef(builtin_name=inject, source_ref=source_ref)
+            makeExpressionBuiltinTypeRef(builtin_name=inject, source_ref=source_ref)
         )
 
 
@@ -132,6 +135,10 @@ def buildFunctionNode(provider, node, source_ref):
 
     assert getKind(node) == "FunctionDef"
 
+    Plugins.onFunctionBodyParsing(
+        provider=provider, function_name=node.name, body=node.body
+    )
+
     function_statement_nodes, function_doc = extractDocFromBody(node)
 
     function_kind, flags = detectFunctionBodyKind(nodes=function_statement_nodes)
@@ -172,10 +179,11 @@ def buildFunctionNode(provider, node, source_ref):
 
         code_body.qualname_provider = provider
 
-        for variable in function_body.getVariables():
+        for variable in function_body.getProvidedVariables():
             code_body.getVariableForReference(variable.getName())
 
-        function_body.setBody(
+        function_body.setChild(
+            "body",
             makeStatementsSequenceFromStatement(
                 statement=StatementReturn(
                     expression=maker_class(
@@ -186,7 +194,7 @@ def buildFunctionNode(provider, node, source_ref):
                     ),
                     source_ref=source_ref,
                 )
-            )
+            ),
         )
 
     decorators = buildNodeList(
@@ -225,7 +233,7 @@ def buildFunctionNode(provider, node, source_ref):
             statement=function_statements_body
         )
 
-    code_body.setBody(function_statements_body)
+    code_body.setChild("body", function_statements_body)
 
     annotations = buildParameterAnnotations(provider, node, source_ref)
 
@@ -244,7 +252,7 @@ def buildFunctionNode(provider, node, source_ref):
     # CPython made these optional, but secretly applies them when it does
     # "class __new__".  We add them earlier, so our optimization will see it.
     if (
-        python_version < 300
+        python_version < 0x300
         and node.name == "__new__"
         and provider.isExpressionClassBody()
     ):
@@ -254,14 +262,14 @@ def buildFunctionNode(provider, node, source_ref):
 
     # Add the "classmethod" decorator to __init_subclass__ methods if not provided.
     if (
-        python_version >= 360
+        python_version >= 0x360
         and node.name == "__init_subclass__"
         and provider.isExpressionClassBody()
     ):
         _injectDecorator(decorators, "classmethod", ("classmethod",), source_ref)
 
     if (
-        python_version >= 370
+        python_version >= 0x370
         and node.name == "__class_getitem__"
         and provider.isExpressionClassBody()
     ):
@@ -280,7 +288,7 @@ def buildFunctionNode(provider, node, source_ref):
         source_ref=source_ref,
     )
 
-    if python_version >= 340:
+    if python_version >= 0x340:
         function_body.qualname_setup = result.getVariableName()
 
     return result
@@ -291,6 +299,10 @@ def buildAsyncFunctionNode(provider, node, source_ref):
     # many details each, pylint: disable=too-many-locals
     assert getKind(node) == "AsyncFunctionDef"
 
+    Plugins.onFunctionBodyParsing(
+        provider=provider, function_name=node.name, body=node.body
+    )
+
     function_statement_nodes, function_doc = extractDocFromBody(node)
 
     function_kind, flags = detectFunctionBodyKind(
@@ -328,7 +340,7 @@ def buildAsyncFunctionNode(provider, node, source_ref):
 
     function_body.qualname_provider = provider
 
-    for variable in creator_function_body.getVariables():
+    for variable in creator_function_body.getProvidedVariables():
         function_body.getVariableForReference(variable.getName())
 
     decorators = buildNodeList(
@@ -356,7 +368,7 @@ def buildAsyncFunctionNode(provider, node, source_ref):
             statement=function_statements_body
         )
 
-    function_body.setBody(function_statements_body)
+    function_body.setChild("body", function_statements_body)
 
     annotations = buildParameterAnnotations(provider, node, source_ref)
 
@@ -382,10 +394,11 @@ def buildAsyncFunctionNode(provider, node, source_ref):
             source_ref=source_ref,
         )
 
-    creator_function_body.setBody(
+    creator_function_body.setChild(
+        "body",
         makeStatementsSequenceFromStatement(
             statement=StatementReturn(expression=creation_node, source_ref=source_ref)
-        )
+        ),
     )
 
     function_creation = ExpressionFunctionCreation(
@@ -424,7 +437,7 @@ def buildParameterKwDefaults(provider, node, function_body, source_ref):
     # Build keyword only arguments default values. We are hiding here, that it
     # is a Python3 only feature.
 
-    if python_version >= 300:
+    if python_version >= 0x300:
         kw_only_names = function_body.getParameters().getKwOnlyParameterNames()
 
         if kw_only_names:
@@ -451,12 +464,12 @@ def buildParameterAnnotations(provider, node, source_ref):
     # Too many branches, because there is too many cases, pylint: disable=too-many-branches
 
     # Build annotations. We are hiding here, that it is a Python3 only feature.
-    if python_version < 300:
+    if python_version < 0x300 or hasPythonFlagNoAnnotations():
         return None
 
     # Starting with Python 3.4, the names of parameters are mangled in
     # annotations as well.
-    if python_version < 340:
+    if python_version < 0x340:
         mangle = lambda variable_name: variable_name
     else:
         mangle = lambda variable_name: mangleName(variable_name, provider)
@@ -483,7 +496,7 @@ def buildParameterAnnotations(provider, node, source_ref):
         else:
             assert False, getKind(arg)
 
-    if python_version >= 380:
+    if python_version >= 0x380:
         for arg in node.args.posonlyargs:
             extractArgAnnotation(arg)
 
@@ -493,7 +506,7 @@ def buildParameterAnnotations(provider, node, source_ref):
     for arg in node.args.kwonlyargs:
         extractArgAnnotation(arg)
 
-    if python_version < 340:
+    if python_version < 0x340:
         if node.args.varargannotation is not None:
             addAnnotation(
                 key=node.args.vararg,
@@ -554,7 +567,7 @@ def _wrapFunctionWithSpecialNestedArgs(
         for element_index, arg_name in enumerate(arg_names):
             if getKind(arg_name) == "Name":
                 arg_var = outer_body.createProvidedVariable(arg_name.id)
-                outer_body.registerProvidedVariable(arg_var)
+                outer_body.getLocalsScope().registerProvidedVariable(arg_var)
 
                 statements.append(
                     StatementAssignmentVariable(
@@ -627,7 +640,8 @@ def _wrapFunctionWithSpecialNestedArgs(
 
     statements.append(StatementReturn(expression=code_body, source_ref=source_ref))
 
-    outer_body.setBody(
+    outer_body.setChild(
+        "body",
         makeStatementsSequenceFromStatement(
             statement=makeTryFinallyStatement(
                 provider=outer_body,
@@ -642,7 +656,7 @@ def _wrapFunctionWithSpecialNestedArgs(
                 source_ref=source_ref,
                 public_exc=False,
             )
-        )
+        ),
     )
 
     return code_body
@@ -694,10 +708,10 @@ def buildFunctionWithParsing(
         ps_name=name,
         ps_normal_args=extractNormalArgs(node.args.args),
         ps_pos_only_args=[extractArg(arg) for arg in node.args.posonlyargs]
-        if python_version >= 380
+        if python_version >= 0x380
         else (),
         ps_kw_only_args=[extractArg(arg) for arg in node.args.kwonlyargs]
-        if python_version >= 300
+        if python_version >= 0x300
         else (),
         ps_list_star_arg=extractArg(node.args.vararg),
         ps_dict_star_arg=extractArg(node.args.kwarg),
@@ -715,6 +729,7 @@ def buildFunctionWithParsing(
         co_name=name,
         co_kind=function_kind,
         co_varnames=parameters.getParameterNames(),
+        co_freevars=(),
         co_argcount=parameters.getArgumentCount(),
         co_posonlyargcount=parameters.getPosOnlyParameterCount(),
         co_kwonlyargcount=parameters.getKwOnlyParameterCount(),
@@ -770,7 +785,7 @@ def addFunctionVariableReleases(function):
         )
 
     if releases:
-        body = function.getBody()
+        body = function.subnode_body
 
         if body.isStatementsFrame():
             body = makeStatementsSequenceFromStatement(statement=body)
@@ -779,4 +794,4 @@ def addFunctionVariableReleases(function):
             provider=function, tried=body, final=releases, source_ref=source_ref
         )
 
-        function.setBody(makeStatementsSequenceFromStatement(statement=body))
+        function.setChild("body", makeStatementsSequenceFromStatement(statement=body))
