@@ -154,7 +154,8 @@ def importFromFolder(logger, module_name, path, must_exist, message):
     from .FileOperations import isPathBelow
 
     if module_name in sys.modules:
-        # May already be loaded, but the wrong one from a pth of clcache
+        # May already be loaded, but the wrong one from a ".pth" file of
+        # clcache that we then don't want to use.
         if module_name != "clcache" or isPathBelow(
             path=path, filename=sys.modules[module_name].__file__
         ):
@@ -195,17 +196,14 @@ def importFromInlineCopy(module_name, must_exist):
         )
     )
 
+    candidate_27 = folder_name + "_27"
+    candidate_35 = folder_name + "_35"
+
     # Use specific versions if needed.
-    if python_version < 0x300:
-        candidate = folder_name + "_27"
-
-        if os.path.exists(candidate):
-            folder_name = candidate
-    elif python_version < 0x360:
-        candidate = folder_name + "_35"
-
-        if os.path.exists(candidate):
-            folder_name = candidate
+    if python_version < 0x300 and os.path.exists(candidate_27):
+        folder_name = candidate_27
+    elif python_version < 0x360 and os.path.exists(candidate_35):
+        folder_name = candidate_35
 
     return importFromFolder(
         module_name=module_name,
