@@ -139,10 +139,16 @@ def getWindowsRelease():
 def isDebianBasedLinux():
     dist_name, base, _dist_version = getLinuxDistribution()
 
-    # False alarm, pylint: disable= superfluous-parens
+    # False alarm, pylint: disable=superfluous-parens
     return (base or dist_name) in ("Debian", "Ubuntu")
 
 
+def isFedoraBasedLinux():
+    dist_name, base, _dist_version = getLinuxDistribution()
+
+    return (base or dist_name) == "Fedora"
+
+
 def isWin32Windows():
     """The Win32 variants of Python does have win32 only, not posix."""
     return os.name == "nt"
