@@ -268,7 +268,7 @@ def addResourceToFile(target_filename, data, resource_kind, lang_id, res_name, l
         try:
             _closeFileWindowsResources(update_handle)
         except OSError as e:
-            if e.errno == 110:
+            if e.errno in (110, 13):
                 logger.warning(
                     """
 Failed to add resources to file %r in attempt %d.
@@ -300,9 +300,11 @@ class WindowsExecutableManifest(object):
         self.tree = TreeXML.fromString(template)
 
     def addResourceToFile(self, filename, logger):
+        manifest_data = TreeXML.toBytes(self.tree, indent=False)
+
         addResourceToFile(
             target_filename=filename,
-            data=TreeXML.toBytes(self.tree),
+            data=manifest_data,
             resource_kind=RT_MANIFEST,
             res_name=1,
             lang_id=0,
@@ -363,10 +365,20 @@ def _getDefaultWindowsExecutableTrustInfo():
 
 
 def getDefaultWindowsExecutableManifest():
+    # Note: Supported OS are lied about by CPython.
     template = (
         """\
 <assembly xmlns="urn:schemas-microsoft-com:asm.v1" manifestVersion="1.0">
   <assemblyIdentity type="win32" name="Mini" version="1.0.0.0"/>
+  <compatibility xmlns="urn:schemas-microsoft-com:compatibility.v1">
+    <application>
+      <supportedOS Id="{e2011457-1546-43c5-a5fe-008deee3d3f0}"/>
+      <supportedOS Id="{35138b9a-5d96-4fbd-8e2d-a2440225f93a}"/>
+      <supportedOS Id="{4a2f28e3-53b9-4441-ba9c-d69d4a4a6e38}"/>
+      <supportedOS Id="{1f676c76-80e1-4239-95bb-83d0f6d0da78}"/>
+      <supportedOS Id="{8e0f7a12-bfb3-4fe8-b9a5-48fd50a15a9a}"/>
+    </application>
+  </compatibility>
   %s
 </assembly>
 """
@@ -644,5 +656,3 @@ def addVersionInfoResource(
         lang_id=0,
         logger=logger,
     )
-
-    return string_values
